/*
 * Copyright (c) 2016 Intel Corporation.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/kernel.h>
#include <errno.h>
#include <zephyr/net/wifi_mgmt.h>
#include <zephyr/net/net_mgmt.h>
#include "ca.h"
#include "client.h"
#include "client_key.h"
#include <zephyr/sys/printk.h>
#include <zephyr/logging/log.h>

LOG_MODULE_REGISTER(wifi, 3);
static volatile bool wifi_connected = false;
struct wifi_eap_config eap_conf = {
	/**  Security type. */
	.type = WIFI_EAP_TYPE_TLS,
	.eap_type_phase1 = WIFI_EAP_TYPE_TLS,
	.eap_type_phase2 = WIFI_EAP_TYPE_TLS, // No inner method needed for TLS
};

struct wifi_enterprise_creds_params enterprise_creds_params = {
	/** CA certification */
	.ca_cert = (char *)ca_pem,
	// /** CA certification length */
	.ca_cert_len = sizeof(ca_pem),
	// /** Client certification */
	.client_cert = (char *)client_pem,
	// /** Client certification length */
	.client_cert_len = sizeof(client_pem),
	// /** Client key */
	.client_key = (char *)client_key,
	// /** Client key length */
	.client_key_len = sizeof(client_key),
};

struct wifi_connect_req_params cnx_params = {
	.ssid = "allen",
	.ssid_length = strlen("allen"),

	/** Private key passwd for enterprise mode */
	.key_passwd = "test",
	/** Private key passwd length, max 128 */
	.key_passwd_length = strlen("test"),
	.eap_identity = "user@example.org",
	/** eap identity length, max 64 */
	.eap_id_length = strlen("user@example.org"),
	/** Password string for EAP. */
	.eap_password = "password",
	/** eap passwd length, max 128 */
	.eap_passwd_length = strlen("password"),
	.channel = WIFI_CHANNEL_ANY,
	.security = WIFI_SECURITY_TYPE_EAP, // WIFI_SECURITY_TYPE_EAP_TTLS_MSCHAPV2 WIFI_SECURITY_TYPE_PSK
	.timeout = SYS_FOREVER_MS,
};

static struct net_mgmt_event_callback wifi_cb;
static void wifi_event_handler(struct net_mgmt_event_callback *cb,
							   uint32_t mgmt_event, struct net_if *iface)
{
	struct wifi_status *status;

	if (mgmt_event == NET_EVENT_WIFI_CONNECT_RESULT)
	{
		status = (struct wifi_status *)cb->info;
		if (status->status)
		{
			LOG_ERR("Wi-Fi connection failed error code: %d", status->status);
			wifi_connected = false;
		}
		else
		{
			LOG_INF("Wi-Fi connect success");
			wifi_connected = true;
		}
	}
	else if (mgmt_event == NET_EVENT_WIFI_DISCONNECT_RESULT)
	{
		LOG_INF("Wi-Fi disconnect");
		wifi_connected = false;
	}
	else if (mgmt_event == NET_EVENT_WIFI_SCAN_RESULT)
	{
		struct wifi_scan_result *scan_result = (struct wifi_scan_result *)cb->info;
		LOG_INF("SSID: %s, Signal: %d dBm, Security: %d",
				scan_result->ssid, scan_result->rssi, scan_result->security);
	}
}

int main(void)
{
	k_sleep(K_SECONDS(1));

	struct net_if *iface = net_if_get_default();
	if (iface == NULL)
	{
		printk("No default network interface found.\n");
	}

	int ret = net_mgmt(NET_REQUEST_WIFI_ENTERPRISE_CREDS, iface, &enterprise_creds_params, sizeof(enterprise_creds_params));
	if (ret)
	{
		printk("Failed to set Wi-Fi enterprise credentials: %d\n", ret);
	}

	k_sleep(K_SECONDS(1));

	net_mgmt_init_event_callback(&wifi_cb, wifi_event_handler,
								 NET_EVENT_WIFI_CONNECT_RESULT |
									 NET_EVENT_WIFI_DISCONNECT_RESULT |
									 NET_EVENT_WIFI_SCAN_RESULT);
	net_mgmt_add_event_callback(&wifi_cb);

	// net_mgmt(NET_REQUEST_WIFI_SCAN, iface, NULL, 0);
	int ret1 = net_mgmt(NET_REQUEST_WIFI_CONNECT, iface, &cnx_params, sizeof(struct wifi_connect_req_params));
	if (ret1)
	{
		printk("Failed to send Wi-Fi connection request! %d\n", ret);
	}
	else
	{
		printk("Wi-Fi connection request sent...\n");
	}

	return 0;
}
